@@ -1,5 +1,14 @@
-package JSON::SL;
+package JSON::SL::Boolean;
 
+use overload
+   "0+"     => sub { ${$_[0]} },
+   "++"     => sub { $_[0] = ${$_[0]} + 1 },
+   "--"     => sub { $_[0] = ${$_[0]} - 1 },
+   fallback => 1;
+1;
+
+
+package JSON::SL;
 use warnings;
 use strict;
 our $VERSION;
@@ -7,7 +16,7 @@ use base qw(Exporter);
 our @EXPORT_OK = qw(decode_json unescape_json_string);
 
 BEGIN {
-    $VERSION = '0.0_2';
+    $VERSION = '0.0_3';
     require XSLoader;
     XSLoader::load(__PACKAGE__, $VERSION);
 }
@@ -404,71 +413,6 @@ already been returned in previous results).
 Only I<complete> objects (i.e. objects which can no longer contain any more data)
 will be returned.
 
-=head3 unescape_settings()
-
-=head3 unescape_settings($character)
-
-=head3 unescape_settings($character, $boolean)
-
-Inspects and modifies the preferences for unescaping JSON strings. JSON allows
-several forms of escape sequences, either via the C<\uXXXX> form, or via a two-
-character 'common' form for specific characters.
-
-=head4 DEFAULT UNESCAPING BEHAVIOR
-
-For C<\uXXXX> escapes, the single or multi-byte representation of the encoded
-character is placed into the resultant string, thus:
-
-    \u0041 becomes A
-
-For JSON structural tokens, the backslash is swallowed and the character following
-it is left as-is. JSON I<requires> that these characters be escaped.
-
-    \"     becomes "
-    \\     becomes \
-    
-
-Additionally, JSON allows the C</> character to be escaped (though it is not
-a JSON structural token, and does not require escaping).
-
-    \/     becomes /
-    
-For certain allowable control and whitespace characters, the escape is translated
-into its corresponding ASCII value, thus:
-
-    \n    becomes chr(0x0A) <LF>
-    \r    becomes chr(0x0D) <CR>
-    \t    becomes chr(0x09) <TAB>
-    \b    becomes chr(0x08) <Backspace>
-    \f    becomes chr(0x0C) <Form Feed>
-
-Any other two-character escape sequence is not allowed, and JSON::SL will croak
-upon encountering it.
-
-By default, all that is allowed to be escaped is also automatically unescaped,
-but this behavior is configurable via the C<unescape_settings>
-
-Called without any arguments, C<unescape_settings> returns a reference to a hash.
-Its keys are valid ASCII characters and its values are booleans
-indicating whether C<JSON::SL> should treat them specially if they follow a C<\>.
-
-Thus, to disable special handling for newlines and tabs:
-    
-    delete @{$json->unescape_settings}{"\t","\n","\r"};
-
-
-If C<unescape_settings> is called with one or two arguments, the first argument
-is taken as the character, and the second argument (if present) is taken as a
-boolean value which the character should be set to:
-
-Check if forward-slashes are unescaped:
-
-    my $fwslash_is_unescaped = $json->unescape_settings("/");
-
-
-Disable handling for C<\uXXXX> sequences:
-
-    $json->unescape_settings("u", 0);
 
 =head2 UTILITY FUNCTIONS
 
@@ -543,7 +487,8 @@ if there are no proper facilities to inspect it?.
 
 =head1 TODO
 
-Work is in progress for a SAX-style interface. Stay tuned
+Work is in progress for a SAX-style interface.
+See L<JSON::SL::Tuba>
 
 =head1 SEE ALSO
 
@@ -563,6 +508,8 @@ L<JSON Specification|http://www.ietf.org/rfc/rfc4627.txt?number=4627>
 L<JSONPointer Specification|http://tools.ietf.org/html/draft-pbryan-zyp-json-pointer-02>
 
 
+L<JSON::SL::Tuba> - Same core with an event-oriented interface, like SAX
+
 =head1 AUTHOR & COPYRIGHT
 
 Copyright (C) 2012 M. Nunberg
